/*
  AIAssistantChat - Interactive AI Assistant for Arduino
  
  This example demonstrates how to use the ArduinoAIAssistant library
  to create an interactive chat interface that communicates with our
  AI service directly through the Serial Monitor.
  
  For WiFi-capable boards (ESP8266/ESP32), this example can connect
  to our AI service API to generate Arduino code and explain code
  directly from the Serial Monitor.
  
  Created: April, 2025
*/

#include <ArduinoAIAssistant.h>

// Create an instance of the AI Assistant
ArduinoAIAssistant aiAssistant;

// For WiFi-capable boards, set your credentials here
#if ARDUINO_AI_WIFI_CAPABLE
const char* ssid = "YOUR_WIFI_SSID";      // <- Replace with your WiFi SSID
const char* password = "YOUR_WIFI_PASS";  // <- Replace with your WiFi password
bool wifiConnected = false;
#endif

void setup() {
  // Initialize serial communication
  Serial.begin(115200);
  
  // Wait for serial port to connect (important for native USB port)
  while (!Serial) {
    ; // wait for serial port to connect
  }
  
  Serial.println("\n\n");
  
  // Initialize the AI Assistant
  aiAssistant.begin();
  
  // For WiFi-capable boards, attempt to connect to WiFi
  #if ARDUINO_AI_WIFI_CAPABLE
  Serial.println("This board supports direct API communication!");
  Serial.println("To connect to WiFi and enable full AI features:");
  Serial.println("1. Edit this sketch to set your WiFi credentials");
  Serial.println("2. Type 'connect' to establish a WiFi connection");
  Serial.println("3. Once connected, you can generate code and get explanations directly");
  
  // Check if credentials were updated
  if (strcmp(ssid, "YOUR_WIFI_SSID") != 0 && strcmp(password, "YOUR_WIFI_PASS") != 0) {
    Serial.println("\nWiFi credentials detected, attempting to connect...");
    wifiConnected = aiAssistant.connectWiFi(ssid, password);
  }
  #endif
  
  // Print instructions
  Serial.println("\nType 'help' for available commands");
  Serial.println("Type 'chat' to enter AI chat mode");
}

void loop() {
  // Handle serial input
  aiAssistant.handleSerialInput();
  
  #if ARDUINO_AI_WIFI_CAPABLE
  // Process 'connect' command for WiFi connection
  if (Serial.available() > 0) {
    String command = Serial.readStringUntil('\n');
    command.trim();
    
    if (command == "connect") {
      if (strcmp(ssid, "YOUR_WIFI_SSID") == 0 || strcmp(password, "YOUR_WIFI_PASS") == 0) {
        Serial.println("Please update the WiFi credentials in the sketch first!");
      } else {
        wifiConnected = aiAssistant.connectWiFi(ssid, password);
      }
    }
  }
  #endif
  
  // Small delay to prevent busy-waiting
  delay(100);
}

/* 
  Example commands to try:
  
  help               - Show help information
  chat               - Enter AI chat mode
  exit               - Exit chat mode
  led                - Generate LED blinking code
  button             - Generate button input code
  
  In chat mode, on WiFi-capable boards:
  generate [description]  - Generate code based on description
  explain [code]         - Get explanation of Arduino code
  
  For example:
  generate a temperature sensor that displays readings on an LCD
  explain void setup() { pinMode(13, OUTPUT); }
*/