/*
  CodeGenerator - Example showing how to use Arduino AI Assistant
  
  This example demonstrates how to use the Serial Monitor to
  communicate with the Arduino AI Assistant and generate code based
  on simple keywords or descriptions.
  
  Visit the web app for the full code generation experience:
  https://426898c2-94bc-4d42-8080-21e328815fab-00-33mrkxttziw7i.riker.replit.dev/
  
  Created: April, 2025
*/

#include <ArduinoAIAssistant.h>

// Create an instance of the AI Assistant
ArduinoAIAssistant aiAssistant;

void setup() {
  // Initialize serial communication
  Serial.begin(9600);
  
  // Wait for serial port to connect (important for native USB port)
  while (!Serial) {
    ; // wait for serial port to connect
  }
  
  // Initialize the AI Assistant
  aiAssistant.begin();
  
  // Print welcome message
  Serial.println("Arduino AI Assistant - Code Generator Example");
  Serial.println("============================================");
  Serial.println("Type one of the following keywords in the Serial Monitor:");
  Serial.println("- 'led': Generate LED blinking code");
  Serial.println("- 'button': Generate button input code");
  Serial.println("- 'sensor': Generate analog sensor reading code");
  Serial.println("- 'servo': Generate servo motor control code");
  Serial.println("- 'help': Show this help message");
  Serial.println("- 'webapp': Show the web app link");
  Serial.println("============================================");
}

void loop() {
  // Check if there's any serial input
  if (Serial.available() > 0) {
    // Read the incoming data
    String input = Serial.readStringUntil('\n');
    input.trim();  // Remove any whitespace
    
    Serial.print("You requested: ");
    Serial.println(input);
    Serial.println();
    
    // Process different keywords
    if (input == "led") {
      generateLEDCode();
    }
    else if (input == "button") {
      generateButtonCode();
    }
    else if (input == "sensor") {
      generateSensorCode();
    }
    else if (input == "servo") {
      generateServoCode();
    }
    else if (input == "help") {
      showHelp();
    }
    else if (input == "webapp") {
      aiAssistant.printWebAppLink();
    }
    else {
      Serial.println("Unknown keyword. Type 'help' to see available options.");
      Serial.println("For custom code generation, visit our web app:");
      aiAssistant.printWebAppLink();
    }
    
    Serial.println("\nEnter another keyword:");
  }
  
  // Small delay to prevent busy-waiting
  delay(100);
}

// Function to generate LED blinking code
void generateLEDCode() {
  Serial.println("// LED Blinking Code:");
  Serial.println("const int ledPin = 13;  // Pin for the LED");
  Serial.println("");
  Serial.println("void setup() {");
  Serial.println("  pinMode(ledPin, OUTPUT);  // Set the LED pin as output");
  Serial.println("}");
  Serial.println("");
  Serial.println("void loop() {");
  Serial.println("  digitalWrite(ledPin, HIGH);  // Turn on the LED");
  Serial.println("  delay(1000);                 // Wait for 1 second");
  Serial.println("  digitalWrite(ledPin, LOW);   // Turn off the LED");
  Serial.println("  delay(1000);                 // Wait for 1 second");
  Serial.println("}");
  
  Serial.println("\nFor more customized LED code (multiple LEDs, patterns, etc.),");
  Serial.println("visit our web app and describe exactly what you need.");
}

// Function to generate button input code
void generateButtonCode() {
  Serial.println("// Button Input Code:");
  Serial.println("const int buttonPin = 2;  // Pin for the button");
  Serial.println("const int ledPin = 13;    // Pin for the LED");
  Serial.println("int buttonState = 0;      // Variable to store button state");
  Serial.println("");
  Serial.println("void setup() {");
  Serial.println("  pinMode(ledPin, OUTPUT);    // Set the LED pin as output");
  Serial.println("  pinMode(buttonPin, INPUT);  // Set the button pin as input");
  Serial.println("}");
  Serial.println("");
  Serial.println("void loop() {");
  Serial.println("  buttonState = digitalRead(buttonPin);  // Read the button state");
  Serial.println("  ");
  Serial.println("  if (buttonState == HIGH) {  // If button is pressed");
  Serial.println("    digitalWrite(ledPin, HIGH);  // Turn on the LED");
  Serial.println("  } else {");
  Serial.println("    digitalWrite(ledPin, LOW);   // Turn off the LED");
  Serial.println("  }");
  Serial.println("}");
  
  Serial.println("\nFor debouncing, pull-up/down resistors, or more complex interactions,");
  Serial.println("visit our web app for customized code.");
}

// Function to generate analog sensor reading code
void generateSensorCode() {
  Serial.println("// Analog Sensor Reading Code:");
  Serial.println("const int sensorPin = A0;  // Analog pin for the sensor");
  Serial.println("int sensorValue = 0;       // Variable to store sensor value");
  Serial.println("");
  Serial.println("void setup() {");
  Serial.println("  Serial.begin(9600);  // Initialize serial communication");
  Serial.println("}");
  Serial.println("");
  Serial.println("void loop() {");
  Serial.println("  sensorValue = analogRead(sensorPin);  // Read the sensor value");
  Serial.println("  Serial.print(\"Sensor value: \");");
  Serial.println("  Serial.println(sensorValue);");
  Serial.println("  delay(1000);  // Wait for 1 second before the next reading");
  Serial.println("}");
  
  Serial.println("\nFor specific sensors (temperature, light, etc.) or processing readings,");
  Serial.println("visit our web app for customized code.");
}

// Function to generate servo motor control code
void generateServoCode() {
  Serial.println("// Servo Motor Control Code:");
  Serial.println("#include <Servo.h>");
  Serial.println("");
  Serial.println("Servo myServo;  // Create a servo object");
  Serial.println("int servoPin = 9;  // Pin for the servo");
  Serial.println("");
  Serial.println("void setup() {");
  Serial.println("  myServo.attach(servoPin);  // Attach the servo to the pin");
  Serial.println("}");
  Serial.println("");
  Serial.println("void loop() {");
  Serial.println("  // Move from 0 to 180 degrees");
  Serial.println("  for (int angle = 0; angle <= 180; angle++) {");
  Serial.println("    myServo.write(angle);  // Set the servo position");
  Serial.println("    delay(15);             // Small delay to allow the servo to move");
  Serial.println("  }");
  Serial.println("  ");
  Serial.println("  // Move from 180 to 0 degrees");
  Serial.println("  for (int angle = 180; angle >= 0; angle--) {");
  Serial.println("    myServo.write(angle);  // Set the servo position");
  Serial.println("    delay(15);             // Small delay to allow the servo to move");
  Serial.println("  }");
  Serial.println("}");
  
  Serial.println("\nFor multiple servos, precise control, or integration with sensors,");
  Serial.println("visit our web app for customized code.");
}

// Function to show help
void showHelp() {
  Serial.println("Arduino AI Assistant - Code Generator Example");
  Serial.println("============================================");
  Serial.println("Type one of the following keywords in the Serial Monitor:");
  Serial.println("- 'led': Generate LED blinking code");
  Serial.println("- 'button': Generate button input code");
  Serial.println("- 'sensor': Generate analog sensor reading code");
  Serial.println("- 'servo': Generate servo motor control code");
  Serial.println("- 'help': Show this help message");
  Serial.println("- 'webapp': Show the web app link");
  Serial.println("============================================");
  Serial.println("For more advanced code generation, visit our web app:");
  aiAssistant.printWebAppLink();
}