// Use window.axios provided by preload script when in renderer process
const axios = typeof window !== 'undefined' && window.axios ? window.axios : require('axios');

/**
 * Coduino AI Assistant API Client
 * Handles communication with the Coduino AI Assistant web service
 * Founded by Houcine Benslimane
 */
class CoduinoAIClient {
  constructor(baseUrl) {
    // Use the current base URL if not specified (default to replit URL format)
    this.baseUrl = baseUrl || window.location.protocol + '//' + window.location.host;
    console.log('API client initialized with URL:', this.baseUrl);
    
    this.axios = axios.create({
      baseURL: this.baseUrl,
      timeout: 30000, // 30 seconds timeout for AI model responses
      headers: {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
      }
    });
  }

  /**
   * Set a custom base URL for the API
   * @param {string} url - The base URL for the API
   */
  setBaseUrl(url) {
    this.baseUrl = url;
    this.axios.defaults.baseURL = url;
  }

  /**
   * Generate Arduino code based on a description
   * @param {string} description - Project description
   * @param {Array<string>} requirements - Optional requirements
   * @param {Array<string>} components - Optional components
   * @param {string} model - Optional AI model to use (gpt-4o, deepseek-chat, claude-3, sonnet)
   * @returns {Promise<Object>} Response object with generated code
   */
  async generateCode(description, requirements = [], components = [], model = null) {
    try {
      const payload = {
        description,
        requirements,
        components
      };
      
      if (model) {
        payload.model = model;
      }
      
      const response = await this.axios.post('/api/generate', payload);
      return response.data;
    } catch (error) {
      console.error('Error generating code:', error);
      throw error;
    }
  }

  /**
   * Get an explanation of Arduino code
   * @param {string} code - Arduino code to explain
   * @param {string} model - Optional AI model to use
   * @returns {Promise<Object>} Explanation object
   */
  async explainCode(code, model = null) {
    try {
      const payload = { code };
      
      if (model) {
        payload.model = model;
      }
      
      const response = await this.axios.post('/api/explain', payload);
      return response.data;
    } catch (error) {
      console.error('Error explaining code:', error);
      throw error;
    }
  }

  /**
   * Check if the API service is healthy
   * @returns {Promise<boolean>} Health status
   */
  async checkHealth() {
    try {
      const response = await this.axios.get('/api/health');
      return response.status === 200;
    } catch (error) {
      console.error('Health check failed:', error);
      return false;
    }
  }

  /**
   * Get available AI models
   * @returns {Promise<Array>} List of available models
   */
  async getAvailableModels() {
    try {
      const response = await this.axios.get('/api/models');
      return response.data;
    } catch (error) {
      console.error('Error fetching models:', error);
      // Return default models if API call fails
      return [
        { id: 'gpt-4o', name: 'GPT-4o (OpenAI)', description: 'Advanced model with code generation capabilities' },
        { id: 'deepseek-chat', name: 'DeepSeek Coder', description: 'Specialized in code generation and understanding' },
        { id: 'claude-3', name: 'Claude 3 (Anthropic)', description: 'Balanced model with good code understanding' },
        { id: 'sonnet', name: 'Sonnet', description: 'Google's AI model for code generation' }
      ];
    }
  }
}

module.exports = CoduinoAIClient;