// Renderer process handles UI interactions
// Access electron API via context bridge
const CoduinoAIClient = window.nodeRequire('./api-client');

// Log renderer start
console.log('Renderer process started');

// DOM Elements
const chatMessages = document.getElementById('chat-messages');
const userInput = document.getElementById('user-input');
const sendButton = document.getElementById('send-button');
const minimizeButton = document.getElementById('minimize-button');
const closeButton = document.getElementById('close-button');
const pinButton = document.getElementById('pin-button');
const modelSelector = document.getElementById('model-selector');

// API client initialization - use current host if running locally
// or override with YOUR_REPLIT_URL for production use
const apiClient = new CoduinoAIClient(window.location.hostname === 'localhost' 
  ? 'http://localhost:5000' 
  : 'https://coduino.replit.app');
let isPinned = true;
let isWaitingForResponse = false;
let selectedModel = 'gpt-4o'; // Default model

// Initialize
document.addEventListener('DOMContentLoaded', () => {
  // Set initial pin state
  updatePinState(isPinned);
  
  // Enable textarea auto-resize
  userInput.addEventListener('input', autoResizeTextarea);
  
  // Populate models from API
  loadAvailableModels();
});

// Event Listeners
userInput.addEventListener('keydown', (e) => {
  if (e.key === 'Enter' && !e.shiftKey) {
    e.preventDefault();
    sendMessage();
  }
});

// Listen for model selection changes
modelSelector.addEventListener('change', (e) => {
  selectedModel = e.target.value;
  console.log(`Model changed to: ${selectedModel}`);
  
  // Add a system message about the model change
  addSystemMessage(`Switched to ${modelSelector.options[modelSelector.selectedIndex].text} model`);
});

sendButton.addEventListener('click', sendMessage);
minimizeButton.addEventListener('click', () => window.electronAPI.sendToMain('minimize-window'));
closeButton.addEventListener('click', () => window.electronAPI.sendToMain('close-window'));
pinButton.addEventListener('click', () => {
  isPinned = !isPinned;
  updatePinState(isPinned);
  window.electronAPI.sendToMain('toggle-always-on-top', isPinned);
});

// Functions
function updatePinState(pinned) {
  if (pinned) {
    pinButton.classList.add('active');
    pinButton.title = 'Unpin window';
  } else {
    pinButton.classList.remove('active');
    pinButton.title = 'Pin window on top';
  }
}

async function sendMessage() {
  const message = userInput.value.trim();
  if (message === '' || isWaitingForResponse) return;
  
  // Add user message to chat
  addMessageToChat('user', message);
  
  // Clear input and resize
  userInput.value = '';
  autoResizeTextarea();
  
  // Show loading indicator
  isWaitingForResponse = true;
  const loadingMsgId = addLoadingIndicator();
  
  try {
    // Send request to API based on message content
    let response;
    
    if (message.toLowerCase().includes('generate code') || 
        message.toLowerCase().includes('create a sketch') ||
        message.toLowerCase().includes('write code')) {
      // Code generation request
      response = await generateArduinoCode(message);
    } else if (message.toLowerCase().includes('explain') || 
              message.toLowerCase().includes('how does this code work')) {
      // Code explanation request
      response = await explainArduinoCode(message);
    } else {
      // General assistance
      response = {
        response: "I can help you generate Arduino code or explain existing code. Try asking something like 'Generate code for a temperature sensor with LCD display' or 'Explain this code: [paste your code here]'."
      };
    }
    
    // Remove loading indicator
    removeLoadingIndicator(loadingMsgId);
    
    // Add assistant response to chat
    addMessageToChat('assistant', response.response || response.explanation || 'Sorry, I encountered an error processing your request.');
    
    // Add code block if present
    if (response.code) {
      addCodeBlockToChat(response.code);
    }
    
  } catch (error) {
    console.error('Error communicating with API:', error);
    
    // Remove loading indicator
    removeLoadingIndicator(loadingMsgId);
    
    // Show error message
    addMessageToChat('assistant', 'Sorry, I encountered an error connecting to the Coduino AI service. Please check your internet connection and try again.');
  }
  
  isWaitingForResponse = false;
  
  // Scroll to bottom
  chatMessages.scrollTop = chatMessages.scrollHeight;
}

function addMessageToChat(sender, content) {
  const messageDiv = document.createElement('div');
  messageDiv.className = `message ${sender}`;
  
  const contentDiv = document.createElement('div');
  contentDiv.className = 'message-content';
  
  // Process content for markdown-like formatting (basic)
  const formattedContent = formatMessageContent(content);
  contentDiv.innerHTML = formattedContent;
  
  messageDiv.appendChild(contentDiv);
  chatMessages.appendChild(messageDiv);
  
  // Scroll to the bottom
  chatMessages.scrollTop = chatMessages.scrollHeight;
}

function addCodeBlockToChat(code) {
  const messageDiv = document.createElement('div');
  messageDiv.className = 'message assistant';
  
  const contentDiv = document.createElement('div');
  contentDiv.className = 'message-content';
  
  const preElement = document.createElement('pre');
  const codeElement = document.createElement('code');
  codeElement.textContent = code;
  
  const copyButton = document.createElement('button');
  copyButton.className = 'copy-code-button';
  copyButton.textContent = 'Copy';
  copyButton.addEventListener('click', () => {
    window.electronAPI.sendToMain('copy-to-clipboard', code);
    copyButton.textContent = 'Copied!';
    setTimeout(() => {
      copyButton.textContent = 'Copy';
    }, 2000);
  });
  
  preElement.appendChild(codeElement);
  preElement.appendChild(copyButton);
  contentDiv.appendChild(preElement);
  messageDiv.appendChild(contentDiv);
  chatMessages.appendChild(messageDiv);
  
  // Scroll to the bottom
  chatMessages.scrollTop = chatMessages.scrollHeight;
}

function formatMessageContent(content) {
  // Convert line breaks to paragraphs
  let formatted = content
    .split('\n\n')
    .map(paragraph => `<p>${paragraph}</p>`)
    .join('');
  
  // Replace single line breaks within paragraphs
  formatted = formatted.replace(/<p>(.*?)\n(.*?)<\/p>/g, '<p>$1<br>$2</p>');
  
  return formatted;
}

function addLoadingIndicator() {
  const messageDiv = document.createElement('div');
  messageDiv.className = 'message assistant';
  const id = 'loading-' + Date.now();
  messageDiv.id = id;
  
  const contentDiv = document.createElement('div');
  contentDiv.className = 'message-content';
  
  const loadingDiv = document.createElement('div');
  loadingDiv.className = 'loading-indicator';
  
  for (let i = 0; i < 3; i++) {
    const dot = document.createElement('div');
    dot.className = 'loading-dot';
    loadingDiv.appendChild(dot);
  }
  
  contentDiv.appendChild(loadingDiv);
  messageDiv.appendChild(contentDiv);
  chatMessages.appendChild(messageDiv);
  
  // Scroll to the bottom
  chatMessages.scrollTop = chatMessages.scrollHeight;
  
  return id;
}

function removeLoadingIndicator(id) {
  const loadingElement = document.getElementById(id);
  if (loadingElement) {
    chatMessages.removeChild(loadingElement);
  }
}

function autoResizeTextarea() {
  userInput.style.height = 'auto';
  userInput.style.height = Math.min(userInput.scrollHeight, 150) + 'px';
}

// Add system message to chat (for model changes, etc.)
function addSystemMessage(message) {
  const messageDiv = document.createElement('div');
  messageDiv.className = 'message system';
  
  const contentDiv = document.createElement('div');
  contentDiv.className = 'message-content system-message';
  contentDiv.textContent = message;
  
  messageDiv.appendChild(contentDiv);
  chatMessages.appendChild(messageDiv);
  
  // Scroll to the bottom
  chatMessages.scrollTop = chatMessages.scrollHeight;
}

// Load available models from API
async function loadAvailableModels() {
  try {
    const models = await apiClient.getAvailableModels();
    
    // Save current selection if any
    const currentSelection = modelSelector.value;
    
    // Clear dropdown
    while (modelSelector.options.length > 0) {
      modelSelector.remove(0);
    }
    
    // Populate with available models
    models.forEach(model => {
      const option = document.createElement('option');
      option.value = model.id;
      option.textContent = model.name;
      option.title = model.description;
      modelSelector.appendChild(option);
    });
    
    // Restore selection or default to first model
    if (currentSelection && Array.from(modelSelector.options).some(opt => opt.value === currentSelection)) {
      modelSelector.value = currentSelection;
    } else {
      // Set to first model if available, otherwise default
      selectedModel = models.length > 0 ? models[0].id : 'gpt-4o';
      modelSelector.value = selectedModel;
    }
  } catch (error) {
    console.error('Failed to load models:', error);
    // Keep default models in the dropdown
  }
}

// API functions
async function generateArduinoCode(description) {
  try {
    const requirements = extractRequirements(description);
    const components = extractComponents(description);
    
    // Pass the selected model
    const response = await apiClient.generateCode(description, requirements, components, selectedModel);
    return response;
  } catch (error) {
    console.error('Error generating code:', error);
    throw error;
  }
}

async function explainArduinoCode(message) {
  // Extract code from the message
  const codeMatch = message.match(/```([\s\S]*?)```/) || 
                    message.match(/explain this code:([\s\S]*)/i) ||
                    message.match(/explain:([\s\S]*)/i);
  
  let code = '';
  if (codeMatch && codeMatch[1]) {
    code = codeMatch[1].trim();
  }
  
  if (!code) {
    return {
      explanation: "I need to see the code to explain it. Please paste your Arduino code and ask me to explain it."
    };
  }
  
  try {
    // Pass the selected model
    const response = await apiClient.explainCode(code, selectedModel);
    return response;
  } catch (error) {
    console.error('Error explaining code:', error);
    throw error;
  }
}

// Helper functions
function extractRequirements(description) {
  // Simple extraction of requirements based on keywords
  const requirements = [];
  const lines = description.split('\n');
  
  for (const line of lines) {
    if (line.toLowerCase().includes('requirement') || 
        line.toLowerCase().includes('should ') || 
        line.toLowerCase().includes('must ')) {
      requirements.push(line.trim());
    }
  }
  
  return requirements;
}

function extractComponents(description) {
  // List of common Arduino components
  const commonComponents = [
    'LED', 'button', 'sensor', 'LCD', 'servo', 'motor', 'relay', 
    'ultrasonic', 'DHT11', 'DHT22', 'temperature', 'humidity', 
    'bluetooth', 'wifi', 'esp8266', 'esp32', 'arduino uno', 
    'arduino nano', 'arduino mega', 'potentiometer', 'accelerometer',
    'gyroscope', 'RFID', 'NFC', 'IR sensor', 'piezo', 'buzzer',
    'joystick', 'LCD display', 'OLED', 'stepper motor', 'breadboard',
    'resistor', 'capacitor', 'transistor', 'diode', 'shift register'
  ];
  
  const components = [];
  const words = description.split(/\s+/);
  
  for (const component of commonComponents) {
    const lowerComponent = component.toLowerCase();
    if (description.toLowerCase().includes(lowerComponent)) {
      components.push(component);
    }
  }
  
  return components;
}